@@ -36,7 +36,7 @@ static void va_glx_error_message(const char *format, ...)
 {
     va_list args;
     va_start(args, format);
-    fprintf(stderr, "[%s] ", PACKAGE_NAME);
+    fprintf(stderr, "libva-glx error: ");
     vfprintf(stderr, format, args);
     va_end(args);
 }
@@ -199,7 +199,7 @@ static int check_tfp_extensions(VADriverContextP ctx)
     if (!check_extension("GL_ARB_texture_non_power_of_two", gl_extensions))
         return 0;
 
-    glx_extensions = glXQueryExtensionsString(ctx->x11_dpy, ctx->x11_screen);
+    glx_extensions = glXQueryExtensionsString((Display *)ctx->native_dpy, ctx->x11_screen);
     if (!check_extension("GLX_EXT_texture_from_pixmap", glx_extensions))
         return 0;
     return 1;
@@ -222,6 +222,14 @@ static int load_tfp_extensions(VADriverContextP ctx)
 {
     VAOpenGLVTableP pOpenGLVTable = gl_get_vtable(ctx);
 
+    pOpenGLVTable->glx_create_pixmap = (PFNGLXCREATEPIXMAPPROC)
+        get_proc_address("glXCreatePixmap");
+    if (!pOpenGLVTable->glx_create_pixmap)
+        return 0;
+    pOpenGLVTable->glx_destroy_pixmap = (PFNGLXDESTROYPIXMAPPROC)
+        get_proc_address("glXDestroyPixmap");
+    if (!pOpenGLVTable->glx_destroy_pixmap)
+        return 0;
     pOpenGLVTable->glx_bind_tex_image = (PFNGLXBINDTEXIMAGEEXTPROC)
         get_proc_address("glXBindTexImageEXT");
     if (!pOpenGLVTable->glx_bind_tex_image)
@@ -332,7 +340,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     if (!cs)
         goto error;
 
-    cs->display = ctx->x11_dpy;
+    cs->display = (Display *)ctx->native_dpy;
     cs->window  = parent ? parent->window : None;
     cs->context = NULL;
 
@@ -349,7 +357,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
             goto choose_fbconfig;
 
         fbconfigs = glXGetFBConfigs(
-            ctx->x11_dpy,
+            (Display *)ctx->native_dpy,
             ctx->x11_screen,
             &n_fbconfigs
         );
@@ -359,7 +367,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
         /* Find out a GLXFBConfig compatible with the parent context */
         for (n = 0; n < n_fbconfigs; n++) {
             status = glXGetFBConfigAttrib(
-                ctx->x11_dpy,
+                (Display *)ctx->native_dpy,
                 fbconfigs[n],
                 GLX_FBCONFIG_ID, &val
             );
@@ -372,7 +380,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     else {
     choose_fbconfig:
         fbconfigs = glXChooseFBConfig(
-            ctx->x11_dpy,
+            (Display *)ctx->native_dpy,
             ctx->x11_screen,
             fbconfig_attrs, &n_fbconfigs
         );
@@ -384,7 +392,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     }
 
     cs->context = glXCreateNewContext(
-        ctx->x11_dpy,
+        (Display *)ctx->native_dpy,
         fbconfigs[n],
         GLX_RGBA_TYPE,
         parent ? parent->context : NULL,
@@ -451,22 +459,23 @@ struct VASurfaceGLX {
 // Create Pixmaps for GLX texture-from-pixmap extension
 static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 {
-    const unsigned int  width      = pSurfaceGLX->width;
-    const unsigned int  height     = pSurfaceGLX->height;
-    Pixmap              pixmap     = None;
-    GLXFBConfig        *fbconfig   = NULL;
-    GLXPixmap           glx_pixmap = None;
-    Window              root_window;
-    XWindowAttributes   wattr;
-    int                *attrib;
-    int                 n_fbconfig_attrs;
+    VAOpenGLVTableP const pOpenGLVTable = gl_get_vtable(ctx);
+    const unsigned int    width         = pSurfaceGLX->width;
+    const unsigned int    height        = pSurfaceGLX->height;
+    Pixmap                pixmap        = None;
+    GLXFBConfig          *fbconfig      = NULL;
+    GLXPixmap             glx_pixmap    = None;
+    Window                root_window;
+    XWindowAttributes     wattr;
+    int                  *attrib;
+    int                   n_fbconfig_attrs;
 
-    root_window = RootWindow(ctx->x11_dpy, ctx->x11_screen);
-    XGetWindowAttributes(ctx->x11_dpy, root_window, &wattr);
+    root_window = RootWindow((Display *)ctx->native_dpy, ctx->x11_screen);
+    XGetWindowAttributes((Display *)ctx->native_dpy, root_window, &wattr);
     if (wattr.depth != 24 && wattr.depth != 32)
         return 0;
     pixmap = XCreatePixmap(
-        ctx->x11_dpy,
+        (Display *)ctx->native_dpy,
         root_window,
         width,
         height,
@@ -500,7 +509,7 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     *attrib++ = GL_NONE;
 
     fbconfig = glXChooseFBConfig(
-        ctx->x11_dpy,
+        (Display *)ctx->native_dpy,
         ctx->x11_screen,
         fbconfig_attrs,
         &n_fbconfig_attrs
@@ -523,8 +532,8 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     *attrib++ = GL_NONE;
 
     x11_trap_errors();
-    glx_pixmap = glXCreatePixmap(
-        ctx->x11_dpy,
+    glx_pixmap = pOpenGLVTable->glx_create_pixmap(
+        (Display *)ctx->native_dpy,
         fbconfig[0],
         pixmap,
         pixmap_attrs
@@ -544,18 +553,20 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 // Destroy Pixmaps used for TFP
 static void destroy_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 {
+    VAOpenGLVTableP const pOpenGLVTable = gl_get_vtable(ctx);
+
     if (pSurfaceGLX->pix_texture) {
         glDeleteTextures(1, &pSurfaceGLX->pix_texture);
         pSurfaceGLX->pix_texture = 0;
     }
 
     if (pSurfaceGLX->glx_pixmap) {
-        glXDestroyPixmap(ctx->x11_dpy, pSurfaceGLX->glx_pixmap);
+        pOpenGLVTable->glx_destroy_pixmap((Display *)ctx->native_dpy, pSurfaceGLX->glx_pixmap);
         pSurfaceGLX->glx_pixmap = None;
     }
 
     if (pSurfaceGLX->pixmap) {
-        XFreePixmap(ctx->x11_dpy, pSurfaceGLX->pixmap);
+        XFreePixmap((Display *)ctx->native_dpy, pSurfaceGLX->pixmap);
         pSurfaceGLX->pixmap = None;
     }
 }
@@ -572,12 +583,12 @@ static int bind_pixmap(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 
     x11_trap_errors();
     pOpenGLVTable->glx_bind_tex_image(
-        ctx->x11_dpy,
+        (Display *)ctx->native_dpy,
         pSurfaceGLX->glx_pixmap,
         GLX_FRONT_LEFT_EXT,
         NULL
     );
-    XSync(ctx->x11_dpy, False);
+    XSync((Display *)ctx->native_dpy, False);
     if (x11_untrap_errors() != 0) {
         va_glx_error_message("failed to bind pixmap\n");
         return 0;
@@ -597,11 +608,11 @@ static int unbind_pixmap(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 
     x11_trap_errors();
     pOpenGLVTable->glx_release_tex_image(
-        ctx->x11_dpy,
+        (Display *)ctx->native_dpy,
         pSurfaceGLX->glx_pixmap,
         GLX_FRONT_LEFT_EXT
     );
-    XSync(ctx->x11_dpy, False);
+    XSync((Display *)ctx->native_dpy, False);
     if (x11_untrap_errors() != 0) {
         va_glx_error_message("failed to release pixmap\n");
         return 0;
@@ -929,13 +940,13 @@ associate_surface(
     status = ctx->vtable.vaPutSurface(
         ctx,
         surface,
-        pSurfaceGLX->pixmap,
+        (void *)pSurfaceGLX->pixmap,
         0, 0, pSurfaceGLX->width, pSurfaceGLX->height,
         0, 0, pSurfaceGLX->width, pSurfaceGLX->height,
         NULL, 0,
         flags
     );
-    XSync(ctx->x11_dpy, False);
+    XSync((Display *)ctx->native_dpy, False);
     if (x11_untrap_errors() != 0)
         return VA_STATUS_ERROR_OPERATION_FAILED;
     if (status != VA_STATUS_SUCCESS)
@@ -1057,14 +1068,8 @@ VAStatus va_glx_init_context(VADriverContextP ctx)
         vtable->vaDestroySurfaceGLX     = vaDestroySurfaceGLX_impl_libva;
         vtable->vaCopySurfaceGLX        = vaCopySurfaceGLX_impl_libva;
 
-        if (!glXQueryVersion(ctx->x11_dpy, &glx_major, &glx_minor))
+        if (!glXQueryVersion((Display *)ctx->native_dpy, &glx_major, &glx_minor))
             return VA_STATUS_ERROR_UNIMPLEMENTED;
-        if (glx_major < 1 || (glx_major == 1 && glx_minor < 3)) { /* GLX 1.3 */
-            va_glx_error_message("GLX version 1.3 expected but only "
-                                 "version %d.%d is available\n",
-                                 glx_major, glx_minor);
-            return VA_STATUS_ERROR_UNIMPLEMENTED;
-        }
 
         if (!check_tfp_extensions(ctx) || !load_tfp_extensions(ctx))
             return VA_STATUS_ERROR_UNIMPLEMENTED;
