@@ -23,12 +23,11 @@
  */
 
 #define _GNU_SOURCE 1
-
-#include "config.h"
+#include "sysdeps.h"
 #include "va.h"
 #include "va_backend.h"
-#include "va_crystalhd.h"
-#include "va_compat.h"
+#include "va_trace.h"
+#include "va_fool.h"
 
 #include <assert.h>
 #include <stdarg.h>
@@ -38,10 +37,7 @@
 #include <dlfcn.h>
 #include <unistd.h>
 
-
 #define DRIVER_INIT_FUNC	"__vaDriverInit_0_31"
-#define DRIVER_INIT_FUNC_0_29	"__vaDriverInit_0_29"
-#define DRIVER_INIT_FUNC_0_30	"__vaDriverInit_0_30"
 
 #define DRIVER_EXTENSION	"_drv_video.so"
 
@@ -53,17 +49,75 @@
 #define CHECK_MAXIMUM(s, ctx, var) if (!va_checkMaximum(ctx->max_##var, #var)) s = VA_STATUS_ERROR_UNKNOWN;
 #define CHECK_STRING(s, ctx, var) if (!va_checkString(ctx->str_##var, #var)) s = VA_STATUS_ERROR_UNKNOWN;
 
-#define TRACE(func) if (va_debug_trace) va_infoMessage("[TR] %s\n", #func);
+extern int trace_flag;
+#define VA_TRACE(trace_func,...)                \
+    if (trace_flag) {                           \
+        trace_func(__VA_ARGS__);                \
+    }
 
-static int va_debug_trace = 0;
+extern int fool_decode;
+extern int fool_encode;
+#define VA_FOOL(fool_func,...)                 \
+    if (fool_decode || fool_encode) {          \
+        ret = fool_func(__VA_ARGS__);          \
+    }
 
-static int vaDisplayIsValid(VADisplay dpy)
+/*
+ * read a config "env" for libva.conf or from environment setting
+ * liva.conf has higher priority
+ * return 0: the "env" is set, and the value is copied into env_value
+ *        1: the env is not set
+ */
+int va_parseConfig(char *env, char *env_value)
 {
-  VADisplayContextP pDisplayContext = (VADisplayContextP)dpy;
-  return pDisplayContext && (pDisplayContext->vadpy_magic == VA_DISPLAY_MAGIC) && pDisplayContext->vaIsValid(pDisplayContext);
+    char *token, *value, *saveptr;
+    char oneline[1024];
+    FILE *fp=NULL;
+
+
+    if (env == NULL)
+        return 1;
+    
+    fp = fopen("/etc/libva.conf", "r");
+    while (fp && (fgets(oneline, 1024, fp) != NULL)) {
+	if (strlen(oneline) == 1)
+	    continue;
+        token = strtok_r(oneline, "=\n", &saveptr);
+	value = strtok_r(NULL, "=\n", &saveptr);
+
+	if (NULL == token || NULL == value)
+	    continue;
+
+        if (strcmp(token, env) == 0) {
+            if (env_value)
+                strncpy(env_value,value, 1024);
+
+            fclose(fp);
+
+            return 0;
+        }
+    }
+    if (fp)
+        fclose(fp);
+
+    /* no setting in config file, use env setting */
+    if (getenv(env)) {
+        if (env_value)
+            strncpy(env_value, getenv(env), 1024);
+
+        return 0;
+    }
+    
+    return 1;
 }
 
-static void va_errorMessage(const char *msg, ...)
+int vaDisplayIsValid(VADisplay dpy)
+{
+    VADisplayContextP pDisplayContext = (VADisplayContextP)dpy;
+    return pDisplayContext && (pDisplayContext->vadpy_magic == VA_DISPLAY_MAGIC) && pDisplayContext->vaIsValid(pDisplayContext);
+}
+
+void va_errorMessage(const char *msg, ...)
 {
     va_list args;
 
@@ -73,7 +127,7 @@ static void va_errorMessage(const char *msg, ...)
     va_end(args);
 }
 
-static void va_infoMessage(const char *msg, ...)
+void va_infoMessage(const char *msg, ...)
 {
     va_list args;
 
@@ -152,8 +206,11 @@ static VAStatus va_openDriver(VADisplay dpy, char *driver_name)
         strncat( driver_path, DRIVER_EXTENSION, strlen(DRIVER_EXTENSION) );
         
         va_infoMessage("Trying to open %s\n", driver_path);
-
+#ifndef ANDROID
         handle = dlopen( driver_path, RTLD_NOW | RTLD_GLOBAL | RTLD_NODELETE );
+#else
+        handle = dlopen( driver_path, RTLD_NOW| RTLD_GLOBAL);
+#endif
         if (!handle)
         {
             /* Don't give errors for non-existing files */
@@ -165,26 +222,7 @@ static VAStatus va_openDriver(VADisplay dpy, char *driver_name)
         else
         {
             VADriverInit init_func;
-            char driver_init_func_sds[32];
-            int compat_version = 0;
-            /* First, try SDS extensions (VDPAU and XvBA backends) */
-            sprintf(driver_init_func_sds, "%s_%d_sds%d",
-                    DRIVER_INIT_FUNC, VA_MICRO_VERSION, VA_SDS_VERSION);
-            init_func = (VADriverInit) dlsym(handle, driver_init_func_sds);
-            if (!init_func)
-            {
-                /* Otherwise, we need the compatibility layer for some buffers */
-                init_func = (VADriverInit) dlsym(handle, DRIVER_INIT_FUNC);
-                compat_version = VA_MINOR_VERSION;
-                if (!init_func) {
-                    init_func = (VADriverInit) dlsym(handle, DRIVER_INIT_FUNC_0_29);
-                    compat_version = 29;
-                }
-                if (!init_func) {
-                    init_func = (VADriverInit) dlsym(handle, DRIVER_INIT_FUNC_0_30);
-                    compat_version = 30;
-                }
-            }
+            init_func = (VADriverInit) dlsym(handle, DRIVER_INIT_FUNC);
             if (!init_func)
             {
                 va_errorMessage("%s has no function %s\n", driver_path, DRIVER_INIT_FUNC);
@@ -192,36 +230,7 @@ static VAStatus va_openDriver(VADisplay dpy, char *driver_name)
             }
             else
             {
-                struct VADriverContext_0_29 ctx_0_29;
-                struct VADriverContext_0_30 ctx_0_30;
-                void *compat_ctx = NULL;
-
-                switch (compat_version) {
-                case 29:
-                    compat_ctx           = &ctx_0_29;
-                    ctx_0_29.pDriverData = NULL;
-                    ctx_0_29.x11_dpy     = ctx->x11_dpy;
-                    ctx_0_29.x11_screen  = ctx->x11_screen;
-                    break;
-                case 30:
-                    compat_ctx           = &ctx_0_30;
-                    ctx_0_30.pDriverData = NULL;
-                    ctx_0_30.x11_dpy     = ctx->x11_dpy;
-                    ctx_0_30.x11_screen  = ctx->x11_screen;
-                    break;
-                case VA_MINOR_VERSION:
-                    compat_ctx           = ctx;
-                    break;
-                default:
-                    ASSERT(compat_version == 0);
-                    vaStatus = VA_STATUS_ERROR_UNKNOWN;
-                    break;
-                }
-
-                vaStatus = (*init_func)(compat_ctx ? compat_ctx : ctx);
-
-                if (VA_STATUS_SUCCESS == vaStatus)
-                    vaStatus = va_compat_init(dpy, compat_version, compat_ctx);
+                vaStatus = (*init_func)(ctx);
 
                 if (VA_STATUS_SUCCESS == vaStatus)
                 {
@@ -361,6 +370,8 @@ const char *vaErrorStr(VAStatus error_status)
             return "the requested function is not implemented";
         case VA_STATUS_ERROR_SURFACE_IN_DISPLAYING:
             return "surface is in displaying (may by overlay)" ;
+        case VA_STATUS_ERROR_INVALID_IMAGE_FORMAT:
+            return "invalid VAImageFormat";
         case VA_STATUS_ERROR_UNKNOWN:
             return "unknown libva error";
     }
@@ -379,7 +390,9 @@ VAStatus vaInitialize (
 
     CHECK_DISPLAY(dpy);
 
-    va_debug_trace = (getenv("LIBVA_DEBUG_TRACE") != NULL);
+    va_TraceInit(dpy);
+
+    va_FoolInit(dpy);
 
     va_infoMessage("libva version %s\n", VA_VERSION_S);
 
@@ -402,15 +415,6 @@ VAStatus vaInitialize (
         vaStatus = va_openDriver(dpy, driver_name);
         va_infoMessage("va_openDriver() returns %d\n", vaStatus);
 
-        if (vaStatus != VA_STATUS_SUCCESS && !driver_name_env)
-        {
-            if (va_hasCrystalHD())
-            {
-                vaStatus = va_openDriver(dpy, "crystalhd");
-                va_infoMessage("va_openDriver() returns %d\n", vaStatus);
-            }
-        }
-
         *major_version = VA_MAJOR_VERSION;
         *minor_version = VA_MINOR_VERSION;
     }
@@ -435,17 +439,19 @@ VAStatus vaTerminate (
   CHECK_DISPLAY(dpy);
   old_ctx = CTX(dpy);
 
-  if (old_ctx->handle)
-  {
+  if (old_ctx->handle) {
       vaStatus = old_ctx->vtable.vaTerminate(old_ctx);
       dlclose(old_ctx->handle);
       old_ctx->handle = NULL;
   }
 
-  va_compat_fini(dpy);
-
   if (VA_STATUS_SUCCESS == vaStatus)
       pDisplayContext->vaDestroy(pDisplayContext);
+
+  va_TraceEnd(dpy);
+
+  va_FoolEnd(dpy);
+
   return vaStatus;
 }
 
@@ -513,7 +519,6 @@ VAStatus vaQueryConfigEntrypoints (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQueryConfigEntrypoints);
   return ctx->vtable.vaQueryConfigEntrypoints ( ctx, profile, entrypoints, num_entrypoints);
 }
 
@@ -529,7 +534,6 @@ VAStatus vaGetConfigAttributes (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaGetConfigAttributes);
   return ctx->vtable.vaGetConfigAttributes ( ctx, profile, entrypoint, attrib_list, num_attribs );
 }
 
@@ -543,7 +547,6 @@ VAStatus vaQueryConfigProfiles (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQueryConfigProfiles);
   return ctx->vtable.vaQueryConfigProfiles ( ctx, profile_list, num_profiles );
 }
 
@@ -557,11 +560,19 @@ VAStatus vaCreateConfig (
 )
 {
   VADriverContextP ctx;
+  VAStatus vaStatus = VA_STATUS_SUCCESS;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCreateConfig);
-  return ctx->vtable.vaCreateConfig ( ctx, profile, entrypoint, attrib_list, num_attribs, config_id );
+  VA_FOOL(va_FoolCreateConfig, dpy, profile, entrypoint, attrib_list, num_attribs, config_id);
+  
+  vaStatus =  ctx->vtable.vaCreateConfig ( ctx, profile, entrypoint, attrib_list, num_attribs, config_id );
+
+  VA_TRACE(va_TraceCreateConfig, dpy, profile, entrypoint, attrib_list, num_attribs, config_id);
+  
+  return vaStatus;
 }
 
 VAStatus vaDestroyConfig (
@@ -573,7 +584,6 @@ VAStatus vaDestroyConfig (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroyConfig);
   return ctx->vtable.vaDestroyConfig ( ctx, config_id );
 }
 
@@ -590,7 +600,6 @@ VAStatus vaQueryConfigAttributes (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQueryConfigAttributes);
   return ctx->vtable.vaQueryConfigAttributes( ctx, config_id, profile, entrypoint, attrib_list, num_attribs);
 }
 
@@ -604,11 +613,19 @@ VAStatus vaCreateSurfaces (
 )
 {
   VADriverContextP ctx;
+  VAStatus vaStatus;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCreateSurfaces);
-  return ctx->vtable.vaCreateSurfaces( ctx, width, height, format, num_surfaces, surfaces );
+  vaStatus = ctx->vtable.vaCreateSurfaces( ctx, width, height, format, num_surfaces, surfaces );
+
+  VA_TRACE(va_TraceCreateSurface, dpy, width, height, format, num_surfaces, surfaces);
+
+  VA_FOOL(va_FoolCreateSurfaces, dpy, width, height, format, num_surfaces, surfaces);
+  
+  return vaStatus;
 }
 
 
@@ -622,7 +639,6 @@ VAStatus vaDestroySurfaces (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroySurfaces);
   return ctx->vtable.vaDestroySurfaces( ctx, surface_list, num_surfaces );
 }
 
@@ -638,12 +654,17 @@ VAStatus vaCreateContext (
 )
 {
   VADriverContextP ctx;
+  VAStatus vaStatus;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCreateContext);
-  return ctx->vtable.vaCreateContext( ctx, config_id, picture_width, picture_height,
+  vaStatus = ctx->vtable.vaCreateContext( ctx, config_id, picture_width, picture_height,
                                       flag, render_targets, num_render_targets, context );
+
+  VA_TRACE(va_TraceCreateContext, dpy, config_id, picture_width, picture_height, flag, render_targets, num_render_targets, context);
+
+  return vaStatus;
 }
 
 VAStatus vaDestroyContext (
@@ -655,7 +676,6 @@ VAStatus vaDestroyContext (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroyContext);
   return ctx->vtable.vaDestroyContext( ctx, context );
 }
 
@@ -672,8 +692,12 @@ VAStatus vaCreateBuffer (
   VADriverContextP ctx;
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
-
-  TRACE(vaCreateBuffer);
+  int ret = 0;
+  
+  VA_FOOL(va_FoolCreateBuffer, dpy, context, type, size, num_elements, data, buf_id);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+  
   return ctx->vtable.vaCreateBuffer( ctx, context, type, size, num_elements, data, buf_id);
 }
 
@@ -687,7 +711,6 @@ VAStatus vaBufferSetNumElements (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaBufferSetNumElements);
   return ctx->vtable.vaBufferSetNumElements( ctx, buf_id, num_elements );
 }
 
@@ -699,11 +722,22 @@ VAStatus vaMapBuffer (
 )
 {
   VADriverContextP ctx;
+  VAStatus va_status;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
+  
+  VA_FOOL(va_FoolMapBuffer, dpy, buf_id, pbuf);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+  
+  va_status = ctx->vtable.vaMapBuffer( ctx, buf_id, pbuf );
 
-  TRACE(vaMapBuffer);
-  return ctx->vtable.vaMapBuffer( ctx, buf_id, pbuf );
+  if (va_status == VA_STATUS_SUCCESS)
+      VA_TRACE(va_TraceMapBuffer, dpy, buf_id, pbuf);
+  
+  return va_status;
 }
 
 VAStatus vaUnmapBuffer (
@@ -714,8 +748,12 @@ VAStatus vaUnmapBuffer (
   VADriverContextP ctx;
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
-
-  TRACE(vaUnmapBuffer);
+  int ret = 0;
+  
+  VA_FOOL(va_FoolUnmapBuffer, dpy, buf_id);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+  
   return ctx->vtable.vaUnmapBuffer( ctx, buf_id );
 }
 
@@ -728,10 +766,25 @@ VAStatus vaDestroyBuffer (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroyBuffer);
   return ctx->vtable.vaDestroyBuffer( ctx, buffer_id );
 }
 
+VAStatus vaBufferInfo (
+    VADisplay dpy,
+    VAContextID context,	/* in */
+    VABufferID buf_id,		/* in */
+    VABufferType *type,		/* out */
+    unsigned int *size,		/* out */
+    unsigned int *num_elements	/* out */
+)
+{
+  VADriverContextP ctx;
+  CHECK_DISPLAY(dpy);
+  ctx = CTX(dpy);
+
+  return ctx->vtable.vaBufferInfo( ctx, context, buf_id, type, size, num_elements );
+}
+
 VAStatus vaBeginPicture (
     VADisplay dpy,
     VAContextID context,
@@ -739,10 +792,17 @@ VAStatus vaBeginPicture (
 )
 {
   VADriverContextP ctx;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaBeginPicture);
+  VA_TRACE(va_TraceBeginPicture, dpy, context, render_target);
+
+  VA_FOOL(va_FoolBeginPicture, dpy, context, render_target);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+
   return ctx->vtable.vaBeginPicture( ctx, context, render_target );
 }
 
@@ -754,10 +814,17 @@ VAStatus vaRenderPicture (
 )
 {
   VADriverContextP ctx;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaRenderPicture);
+  VA_FOOL(va_FoolRenderPicture, dpy, context, buffers, num_buffers);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+
+  VA_TRACE(va_TraceRenderPicture, dpy, context, buffers, num_buffers);
+
   return ctx->vtable.vaRenderPicture( ctx, context, buffers, num_buffers );
 }
 
@@ -766,12 +833,24 @@ VAStatus vaEndPicture (
     VAContextID context
 )
 {
+  VAStatus va_status;
   VADriverContextP ctx;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaEndPicture);
-  return ctx->vtable.vaEndPicture( ctx, context );
+  VA_FOOL(va_FoolEndPicture, dpy, context);
+  if (ret) {
+      VA_TRACE(va_TraceEndPicture, dpy, context);
+      return VA_STATUS_SUCCESS;
+  }
+  
+  va_status = ctx->vtable.vaEndPicture( ctx, context );
+  
+  VA_TRACE(va_TraceEndPicture, dpy, context);
+
+  return va_status;
 }
 
 VAStatus vaSyncSurface (
@@ -779,12 +858,21 @@ VAStatus vaSyncSurface (
     VASurfaceID render_target
 )
 {
+  VAStatus va_status;
   VADriverContextP ctx;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSyncSurface);
-  return ctx->vtable.vaSyncSurface( ctx, render_target );
+  VA_FOOL(va_FoolSyncSurface, dpy, render_target);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+  
+  va_status = ctx->vtable.vaSyncSurface( ctx, render_target );
+  VA_TRACE(va_TraceSyncSurface, dpy, render_target);
+
+  return va_status;
 }
 
 VAStatus vaQuerySurfaceStatus (
@@ -793,39 +881,35 @@ VAStatus vaQuerySurfaceStatus (
     VASurfaceStatus *status	/* out */
 )
 {
+  VAStatus va_status;
   VADriverContextP ctx;
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQuerySurfaceStatus);
-  return ctx->vtable.vaQuerySurfaceStatus( ctx, render_target, status );
+  va_status = ctx->vtable.vaQuerySurfaceStatus( ctx, render_target, status );
+
+  VA_TRACE(va_TraceQuerySurfaceStatus, dpy, render_target, status);
+
+  return va_status;
 }
 
-VAStatus vaPutSurface (
-    VADisplay dpy,
-    VASurfaceID surface,
-    Drawable draw, /* X Drawable */
-    short srcx,
-    short srcy,
-    unsigned short srcw,
-    unsigned short srch,
-    short destx,
-    short desty,
-    unsigned short destw,
-    unsigned short desth,
-    VARectangle *cliprects, /* client supplied clip list */
-    unsigned int number_cliprects, /* number of clip rects in the clip list */
-    unsigned int flags /* de-interlacing flags */
+VAStatus vaQuerySurfaceError (
+	VADisplay dpy,
+	VASurfaceID surface,
+	VAStatus error_status,
+	void **error_info /*out*/
 )
 {
+  VAStatus va_status;
   VADriverContextP ctx;
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaPutSurface);
-  return ctx->vtable.vaPutSurface( ctx, surface, draw, srcx, srcy, srcw, srch,
-                                   destx, desty, destw, desth,
-                                   cliprects, number_cliprects, flags );
+  va_status = ctx->vtable.vaQuerySurfaceError( ctx, surface, error_status, error_info );
+
+  VA_TRACE(va_TraceQuerySurfaceError, dpy, surface, error_status, error_info);
+
+  return va_status;
 }
 
 /* Get maximum number of image formats supported by the implementation */
@@ -849,7 +933,6 @@ VAStatus vaQueryImageFormats (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQueryImageFormats);
   return ctx->vtable.vaQueryImageFormats ( ctx, format_list, num_formats);
 }
 
@@ -873,7 +956,6 @@ VAStatus vaCreateImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCreateImage);
   return ctx->vtable.vaCreateImage ( ctx, format, width, height, image);
 }
 
@@ -889,7 +971,6 @@ VAStatus vaDestroyImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroyImage);
   return ctx->vtable.vaDestroyImage ( ctx, image);
 }
 
@@ -903,7 +984,6 @@ VAStatus vaSetImagePalette (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSetImagePalette);
   return ctx->vtable.vaSetImagePalette ( ctx, image, palette);
 }
 
@@ -925,7 +1005,6 @@ VAStatus vaGetImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaGetImage);
   return ctx->vtable.vaGetImage ( ctx, surface, x, y, width, height, image);
 }
 
@@ -951,7 +1030,6 @@ VAStatus vaPutImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaPutImage);
   return ctx->vtable.vaPutImage ( ctx, surface, image, src_x, src_y, src_width, src_height, dest_x, dest_y, dest_width, dest_height );
 }
 
@@ -996,7 +1074,6 @@ VAStatus vaDeriveImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDeriveImage);
   return ctx->vtable.vaDeriveImage ( ctx, surface, image );
 }
 
@@ -1027,10 +1104,15 @@ VAStatus vaQuerySubpictureFormats (
 )
 {
   VADriverContextP ctx;
+  int ret = 0;
+  
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
-
-  TRACE(vaQuerySubpictureFormats);
+  
+  VA_FOOL(va_FoolQuerySubpictureFormats, dpy, format_list, flags, num_formats);
+  if (ret)
+      return VA_STATUS_SUCCESS;
+  
   return ctx->vtable.vaQuerySubpictureFormats ( ctx, format_list, flags, num_formats);
 }
 
@@ -1047,7 +1129,6 @@ VAStatus vaCreateSubpicture (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCreateSubpicture);
   return ctx->vtable.vaCreateSubpicture ( ctx, image, subpicture );
 }
 
@@ -1063,7 +1144,6 @@ VAStatus vaDestroySubpicture (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDestroySubpicture);
   return ctx->vtable.vaDestroySubpicture ( ctx, subpicture);
 }
 
@@ -1077,7 +1157,6 @@ VAStatus vaSetSubpictureImage (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSetSubpictureImage);
   return ctx->vtable.vaSetSubpictureImage ( ctx, subpicture, image);
 }
 
@@ -1098,7 +1177,6 @@ VAStatus vaSetSubpictureChromakey (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSetSubpictureChromakey);
   return ctx->vtable.vaSetSubpictureChromakey ( ctx, subpicture, chromakey_min, chromakey_max, chromakey_mask );
 }
 
@@ -1118,7 +1196,6 @@ VAStatus vaSetSubpictureGlobalAlpha (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSetSubpictureGlobalAlpha);
   return ctx->vtable.vaSetSubpictureGlobalAlpha ( ctx, subpicture, global_alpha );
 }
 
@@ -1153,7 +1230,6 @@ VAStatus vaAssociateSubpicture (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaAssociateSubpicture);
   return ctx->vtable.vaAssociateSubpicture ( ctx, subpicture, target_surfaces, num_surfaces, src_x, src_y, src_width, src_height, dest_x, dest_y, dest_width, dest_height, flags );
 }
 
@@ -1171,7 +1247,6 @@ VAStatus vaDeassociateSubpicture (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaDeassociateSubpicture);
   return ctx->vtable.vaDeassociateSubpicture ( ctx, subpicture, target_surfaces, num_surfaces );
 }
 
@@ -1181,10 +1256,16 @@ int vaMaxNumDisplayAttributes (
     VADisplay dpy
 )
 {
+  int tmp;
+    
   if( !vaDisplayIsValid(dpy) )
       return 0;
   
-  return CTX(dpy)->max_display_attributes;
+  tmp = CTX(dpy)->max_display_attributes;
+
+  VA_TRACE(va_TraceMaxNumDisplayAttributes, dpy, tmp);
+  
+  return tmp;
 }
 
 /* 
@@ -1203,8 +1284,14 @@ VAStatus vaQueryDisplayAttributes (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaQueryDisplayAttributes);
-  return ctx->vtable.vaQueryDisplayAttributes ( ctx, attr_list, num_attributes );
+  VAStatus va_status;
+  
+  va_status = ctx->vtable.vaQueryDisplayAttributes ( ctx, attr_list, num_attributes );
+
+  VA_TRACE(va_TraceQueryDisplayAttributes, dpy, attr_list, num_attributes);
+
+  return va_status;
+  
 }
 
 /* 
@@ -1223,8 +1310,13 @@ VAStatus vaGetDisplayAttributes (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaGetDisplayAttributes);
-  return ctx->vtable.vaGetDisplayAttributes ( ctx, attr_list, num_attributes );
+  VAStatus va_status;
+  
+  va_status = ctx->vtable.vaGetDisplayAttributes ( ctx, attr_list, num_attributes );
+
+  VA_TRACE(va_TraceGetDisplayAttributes, dpy, attr_list, num_attributes);
+  
+  return va_status;
 }
 
 /* 
@@ -1243,75 +1335,13 @@ VAStatus vaSetDisplayAttributes (
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaSetDisplayAttributes);
+  VA_TRACE(va_TraceSetDisplayAttributes, dpy, attr_list, num_attributes);
+
+  
   return ctx->vtable.vaSetDisplayAttributes ( ctx, attr_list, num_attributes );
 }
 
-/* Wrap a CI (camera imaging) frame as a VA surface to share captured video between camear
- * and VA encode. With frame_id, VA driver need to call CI interfaces to get the information
- * of the frame, and to determine if the frame can be wrapped as a VA surface
- *
- * Application should make sure the frame is idle before the frame is passed into VA stack
- * and also a vaSyncSurface should be called before application tries to access the frame
- * from CI stack
- */
-VAStatus vaCreateSurfaceFromCIFrame (
-    VADisplay dpy,
-    unsigned long frame_id,
-    VASurfaceID *surface	/* out */
-)
-{
-  VADriverContextP ctx;
-  CHECK_DISPLAY(dpy);
-  ctx = CTX(dpy);
-
-  TRACE(vaCreateSurfacesFromCIFrame);
-
-  if (ctx->vtable.vaCreateSurfaceFromCIFrame)
-      return ctx->vtable.vaCreateSurfaceFromCIFrame( ctx, frame_id, surface );
-  else
-      return VA_STATUS_ERROR_UNKNOWN;
-}
-
-
-/* Wrap a V4L2 buffer as a VA surface, so that V4L2 camera, VA encode
- * can share the data without copy
- * The VA driver should query the camera device from v4l2_fd to see
- * if camera device memory/buffer can be wrapped into a VA surface
- * Buffer information is passed in by v4l2_fmt and v4l2_buf structure,
- * VA driver also needs do further check if the buffer can meet encode
- * hardware requirement, such as dimension, fourcc, stride, etc
- *
- * Application should make sure the buffer is idle before the frame into VA stack
- * and also a vaSyncSurface should be called before application tries to access the frame
- * from V4L2 stack
- */
-VAStatus vaCreateSurfaceFromV4L2Buf(
-    VADisplay dpy,
-    int v4l2_fd,         /* file descriptor of V4L2 device */
-    struct v4l2_format *v4l2_fmt,       /* format of V4L2 */
-    struct v4l2_buffer *v4l2_buf,       /* V4L2 buffer */
-    VASurfaceID *surface	       /* out */
-)
-{
-  VADriverContextP ctx;
-  CHECK_DISPLAY(dpy);
-  ctx = CTX(dpy);
-
-  TRACE(vtable.vaCreateSurfaceFromV4L2Buf);
-
-  if (ctx->vtable.vaCreateSurfaceFromV4L2Buf) 
-      return ctx->vtable.vaCreateSurfaceFromV4L2Buf( ctx, v4l2_fd, v4l2_fmt, v4l2_buf, surface );
-  else
-      return VA_STATUS_ERROR_UNKNOWN;
-}
-
-/* It is a debug interface, and isn't exported in core VAAPI
- * It is used to dump surface data into system memory
- * Application should explicitly call free to release the buffer memory
- */
-
-VAStatus vaCopySurfaceToBuffer(VADisplay dpy,
+VAStatus vaLockSurface(VADisplay dpy,
     VASurfaceID surface,
     unsigned int *fourcc, /* following are output argument */
     unsigned int *luma_stride,
@@ -1320,6 +1350,7 @@ VAStatus vaCopySurfaceToBuffer(VADisplay dpy,
     unsigned int *luma_offset,
     unsigned int *chroma_u_offset,
     unsigned int *chroma_v_offset,
+    unsigned int *buffer_name,
     void **buffer 
 )
 {
@@ -1327,9 +1358,17 @@ VAStatus vaCopySurfaceToBuffer(VADisplay dpy,
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
 
-  TRACE(vaCopySurfaceToBuffer);
-  if (ctx->vtable.vaCopySurfaceToBuffer)
-      return ctx->vtable.vaCopySurfaceToBuffer( ctx, surface, fourcc, luma_stride, chroma_u_stride, chroma_v_stride, luma_offset, chroma_u_offset, chroma_v_offset, buffer);
-  else
-      return VA_STATUS_ERROR_UNKNOWN;
+  return ctx->vtable.vaLockSurface( ctx, surface, fourcc, luma_stride, chroma_u_stride, chroma_v_stride, luma_offset, chroma_u_offset, chroma_v_offset, buffer_name, buffer);
+}
+
+
+VAStatus vaUnlockSurface(VADisplay dpy,
+    VASurfaceID surface
+)
+{
+  VADriverContextP ctx;
+  CHECK_DISPLAY(dpy);
+  ctx = CTX(dpy);
+
+  return ctx->vtable.vaUnlockSurface( ctx, surface );
 }
