@@ -30,10 +30,11 @@
 #define _VA_BACKEND_H_
 
 #include <va/va.h>
+#ifndef ANDROID
 #include <X11/Xlib.h>
+#endif
 #include <linux/videodev2.h>
 
-
 typedef struct VADriverContext *VADriverContextP;
 typedef struct VADisplayContext *VADisplayContextP;
 
@@ -177,10 +178,17 @@ struct VADriverVTable
 		VASurfaceStatus *status	/* out */
 	);
 
+	VAStatus (*vaQuerySurfaceError) (
+		VADriverContextP ctx,
+		VASurfaceID render_target,
+        VAStatus error_status,
+        void **error_info /*out*/
+	);
+
 	VAStatus (*vaPutSurface) (
     		VADriverContextP ctx,
 		VASurfaceID surface,
-		Drawable draw, /* X Drawable */
+		void* draw, /* Drawable of window system */
 		short srcx,
 		short srcy,
 		unsigned short srcw,
@@ -337,23 +345,18 @@ struct VADriverVTable
                 int num_attributes
         );
 
-        /* device specific */
-	VAStatus (*vaCreateSurfaceFromCIFrame) (
-		VADriverContextP ctx,
-		unsigned long frame_id,
-		VASurfaceID *surface		/* out */
-	);
-    
-    
-        VAStatus (*vaCreateSurfaceFromV4L2Buf) (
-		VADriverContextP ctx,
-                int v4l2_fd,         /* file descriptor of V4L2 device */
-                struct v4l2_format *v4l2_fmt,       /* format of V4L2 */
-                struct v4l2_buffer *v4l2_buf,       /* V4L2 buffer */
-                VASurfaceID *surface	           /* out */
+        /* used by va trace */        
+        VAStatus (*vaBufferInfo) (
+                   VADriverContextP ctx,
+                   VAContextID context, /* in */
+                   VABufferID buf_id, /* in */
+                   VABufferType *type,    /* out */
+                   unsigned int *size,    /* out */
+                   unsigned int *num_elements /* out */
         );
-    
-        VAStatus (*vaCopySurfaceToBuffer) (
+
+        /* lock/unlock surface for external access */    
+        VAStatus (*vaLockSurface) (
 		VADriverContextP ctx,
                 VASurfaceID surface,
                 unsigned int *fourcc, /* out  for follow argument */
@@ -363,7 +366,17 @@ struct VADriverVTable
                 unsigned int *luma_offset,
                 unsigned int *chroma_u_offset,
                 unsigned int *chroma_v_offset,
-                void **buffer
+                unsigned int *buffer_name, /* if it is not NULL, assign the low lever
+                                            * surface buffer name
+                                            */
+                void **buffer /* if it is not NULL, map the surface buffer for
+                                * CPU access
+                                */
+        );
+    
+        VAStatus (*vaUnlockSurface) (
+		VADriverContextP ctx,
+                VASurfaceID surface
         );
 
         /* Optional: GLX support hooks */
@@ -374,8 +387,9 @@ struct VADriverContext
 {
     void *pDriverData;
     struct VADriverVTable vtable;
+    void *vtable_tpi; /* the structure is malloc-ed */
 
-    Display *x11_dpy;
+    void *native_dpy;
     int x11_screen;
     int version_major;
     int version_minor;
@@ -390,8 +404,7 @@ struct VADriverContext
     void *handle;			/* dlopen handle */
     
     void *dri_state;
-    void *glx;                          /* opaque for GLX code */
-    void *compat;			/* opaque for compat code */
+    void *glx;				/* opaque for GLX code */
 };
 
 #define VA_DISPLAY_MAGIC 0x56414430 /* VAD0 */
@@ -415,12 +428,11 @@ struct VADisplayContext
 	char **driver_name
     );
 
-    void *opaque;                       /* opaque for display extensions (e.g. GLX) */
+    void *opaque; /* opaque for display extensions (e.g. GLX) */
 };
 
 typedef VAStatus (*VADriverInit) (
     VADriverContextP driver_context
 );
 
-
 #endif /* _VA_BACKEND_H_ */
